class NI_LOGGER

  # LOG_KEYS = [:context, :action_id, :message]
  SEVERITY_TYPES = [:info, :warn, :error, :debug, :unknown, :fatal]

  @defaults = {
      context: :ni_log,
      # action_id: nil,
      # server_name: nil,
      # method: nil,
      # params: nil,
      message: 'Empty message',
      # stack_trace: nil
  }

  @options = {}.merge(@defaults)

  SEVERITY_TYPES.each do |severity_type|
    define_singleton_method severity_type do |context, server, action_id, method, params, message, stack_trace|
      self.log({severity: severity_type, context: context, server: server, action_id: action_id, method: method, params: params, message: message, stack_trace: stack_trace})
    end
  end

  @context_loggers = {}

  def self.setup(options)
    @options = (@options || {}).merge(options)
  end

  def self.options
    return @options
  end

  def self.logs_folder
    # TODO: set the path from config
    @logs_folder ||= "#{Rails.root}/log"
  end


  private

  def self.log_file_path(current_context)
    puts 'current_context:'
    puts current_context
    file_name = current_context.to_s.downcase.gsub(/ /, '_').gsub(/[^0-9a-z_]/, '')
    return "#{logs_folder}/#{file_name}.log"
  end

  def self.log(params)
    params_with_defaults = options.merge(params.reject{|k,v| v.nil?})

    puts 'params_with_defaults:'
    puts params_with_defaults

    current_context = params_with_defaults[:context]
    current_severity = params_with_defaults[:severity]

    logger_instance = logger_of_context(current_context)

    logger_instance.send current_severity.to_sym, params_with_defaults.except(:context, :severity)
    Rails.logger.send current_severity.to_sym, params_with_defaults.except(:severity)
  end

  def self.logger_of_context(current_context)
    @context_loggers[current_context] ||= Logger.new(log_file_path(current_context))
  end

end